package sun.security.provider.certpath;

import sun.security.x509.X500Name;
import sun.security.x509.AuthorityKeyIdentifierExtension;
import sun.security.x509.X509CertImpl;
import java.util.Comparator;
import java.util.LinkedList;
import java.security.PublicKey;
import java.security.InvalidKeyException;
import java.util.ArrayList;
import java.security.GeneralSecurityException;
import java.security.cert.PKIXReason;
import sun.security.x509.PKIXExtensions;
import java.security.cert.PKIXCertPathChecker;
import java.security.cert.CertPathValidatorException;
import java.util.Collections;
import sun.security.x509.AccessDescription;
import sun.security.x509.AuthorityInfoAccessExtension;
import java.io.IOException;
import java.security.cert.CertificateException;
import java.security.cert.CertStoreException;
import java.util.TreeSet;
import java.util.Collection;
import java.security.cert.CertStore;
import java.util.List;
import java.util.Iterator;
import java.util.HashSet;
import java.security.cert.X509CertSelector;
import java.security.cert.TrustAnchor;
import javax.security.auth.x500.X500Principal;
import java.security.cert.X509Certificate;
import java.util.Set;
import sun.security.util.Debug;

class ForwardBuilder extends Builder
{
    private static final Debug debug;
    private final Set<X509Certificate> trustedCerts;
    private final Set<X500Principal> trustedSubjectDNs;
    private final Set<TrustAnchor> trustAnchors;
    private X509CertSelector eeSelector;
    private AdaptableX509CertSelector caSelector;
    private X509CertSelector caTargetSelector;
    TrustAnchor trustAnchor;
    private boolean searchAllCertStores;
    
    ForwardBuilder(PKIX.BuilderParams builderParams, boolean searchAllCertStores) {
        super(builderParams);
        this.searchAllCertStores = true;
        this.trustAnchors = builderParams.trustAnchors();
        this.trustedCerts = new HashSet(this.trustAnchors.size());
        this.trustedSubjectDNs = new HashSet(this.trustAnchors.size());
        for (TrustAnchor trustAnchor : this.trustAnchors) {
            X509Certificate trustedCert = trustAnchor.getTrustedCert();
            if (trustedCert != null) {
                this.trustedCerts.add(trustedCert);
                this.trustedSubjectDNs.add(trustedCert.getSubjectX500Principal());
            }
            else {
                this.trustedSubjectDNs.add(trustAnchor.getCA());
            }
        }
        this.searchAllCertStores = searchAllCertStores;
    }
    
    @Override
    Collection<X509Certificate> getMatchingCerts(State state, List<CertStore> list) throws CertStoreException, CertificateException, IOException {
        if (ForwardBuilder.debug != null) {
            ForwardBuilder.debug.println("ForwardBuilder.getMatchingCerts()...");
        }
        ForwardState forwardState = (ForwardState)state;
        TreeSet<X509Certificate> set = new TreeSet(new ForwardBuilder.PKIXCertComparator(this.trustedSubjectDNs, forwardState.cert));
        if (forwardState.isInitial()) {
            this.getMatchingEECerts(forwardState, list, set);
        }
        this.getMatchingCACerts(forwardState, list, set);
        return set;
    }
    
    private void getMatchingEECerts(ForwardState forwardState, List<CertStore> list, Collection<X509Certificate> collection) throws IOException {
        if (ForwardBuilder.debug != null) {
            ForwardBuilder.debug.println("ForwardBuilder.getMatchingEECerts()...");
        }
        if (this.eeSelector == null) {
            (this.eeSelector = (X509CertSelector)this.targetCertConstraints.clone()).setCertificateValid(this.buildParams.date());
            if (this.buildParams.explicitPolicyRequired()) {
                this.eeSelector.setPolicy(this.getMatchingPolicies());
            }
            this.eeSelector.setBasicConstraints(-2);
        }
        this.addMatchingCerts(this.eeSelector, list, collection, this.searchAllCertStores);
    }
    
    private void getMatchingCACerts(ForwardState forwardState, List<CertStore> list, Collection<X509Certificate> collection) throws IOException {
        if (ForwardBuilder.debug != null) {
            ForwardBuilder.debug.println("ForwardBuilder.getMatchingCACerts()...");
        }
        int size = collection.size();
        X509CertSelector x509CertSelector;
        if (forwardState.isInitial()) {
            if (this.targetCertConstraints.getBasicConstraints() == -2) {
                return;
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("ForwardBuilder.getMatchingCACerts(): the target is a CA");
            }
            if (this.caTargetSelector == null) {
                this.caTargetSelector = (X509CertSelector)this.targetCertConstraints.clone();
                if (this.buildParams.explicitPolicyRequired()) {
                    this.caTargetSelector.setPolicy(this.getMatchingPolicies());
                }
            }
            x509CertSelector = this.caTargetSelector;
        }
        else {
            if (this.caSelector == null) {
                this.caSelector = new AdaptableX509CertSelector();
                if (this.buildParams.explicitPolicyRequired()) {
                    this.caSelector.setPolicy(this.getMatchingPolicies());
                }
            }
            this.caSelector.setSubject(forwardState.issuerDN);
            CertPathHelper.setPathToNames(this.caSelector, forwardState.subjectNamesTraversed);
            this.caSelector.setValidityPeriod(forwardState.cert.getNotBefore(), forwardState.cert.getNotAfter());
            x509CertSelector = this.caSelector;
        }
        x509CertSelector.setBasicConstraints(-1);
        for (X509Certificate cert : this.trustedCerts) {
            if (x509CertSelector.match(cert)) {
                if (ForwardBuilder.debug != null) {
                    ForwardBuilder.debug.println("ForwardBuilder.getMatchingCACerts: found matching trust anchor.\n  SN: " + Debug.toHexString(cert.getSerialNumber()) + "\n  Subject: " + cert.getSubjectX500Principal() + "\n  Issuer: " + cert.getIssuerX500Principal());
                }
                if (collection.add(cert) && !this.searchAllCertStores) {
                    return;
                }
                continue;
            }
            else {
                continue;
            }
        }
        x509CertSelector.setCertificateValid(this.buildParams.date());
        x509CertSelector.setBasicConstraints(forwardState.traversedCACerts);
        if ((forwardState.isInitial() || this.buildParams.maxPathLength() == -1 || this.buildParams.maxPathLength() > forwardState.traversedCACerts) && this.addMatchingCerts(x509CertSelector, list, collection, this.searchAllCertStores) && !this.searchAllCertStores) {
            return;
        }
        if (!forwardState.isInitial() && Builder.USE_AIA) {
            AuthorityInfoAccessExtension authorityInfoAccessExtension = forwardState.cert.getAuthorityInfoAccessExtension();
            if (authorityInfoAccessExtension != null) {
                this.getCerts(authorityInfoAccessExtension, collection);
            }
        }
        if (ForwardBuilder.debug != null) {
            ForwardBuilder.debug.println("ForwardBuilder.getMatchingCACerts: found " + (collection.size() - size) + " CA certs");
        }
        return;
    }
    
    private boolean getCerts(AuthorityInfoAccessExtension authorityInfoAccessExtension, Collection<X509Certificate> collection) {
        if (!Builder.USE_AIA) {
            return false;
        }
        List<AccessDescription> accessDescriptions = authorityInfoAccessExtension.getAccessDescriptions();
        if (accessDescriptions == null || accessDescriptions.isEmpty()) {
            return false;
        }
        boolean n = false;
        Iterator<Object> iterator = accessDescriptions.iterator();
        while (iterator.hasNext()) {
            CertStore instance = URICertStore.getInstance((AccessDescription)iterator.next());
            if (instance != null) {
                try {
                    if (collection.addAll(instance.getCertificates(this.caSelector))) {
                        n = true;
                        if (!this.searchAllCertStores) {
                            return true;
                        }
                        continue;
                    }
                    else {
                        continue;
                    }
                }
                catch (CertStoreException ex) {
                    if (ForwardBuilder.debug != null) {
                        ForwardBuilder.debug.println("exception getting certs from CertStore:");
                        ex.printStackTrace();
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                continue;
            }
        }
        return n;
    }
    
    @Override
    void verifyCert(X509Certificate x509Certificate, State state, List<X509Certificate> list) throws GeneralSecurityException {
        if (ForwardBuilder.debug != null) {
            ForwardBuilder.debug.println("ForwardBuilder.verifyCert(SN: " + Debug.toHexString(x509Certificate.getSerialNumber()) + "\n  Issuer: " + x509Certificate.getIssuerX500Principal() + ")\n  Subject: " + x509Certificate.getSubjectX500Principal() + ")");
        }
        ForwardState forwardState = (ForwardState)state;
        forwardState.untrustedChecker.check(x509Certificate, Collections.emptySet());
        if (list != null) {
            Iterator<Object> iterator = list.iterator();
            while (iterator.hasNext()) {
                if (x509Certificate.equals((Object)iterator.next())) {
                    if (ForwardBuilder.debug != null) {
                        ForwardBuilder.debug.println("loop detected!!");
                    }
                    throw new CertPathValidatorException("loop detected");
                }
                else {
                    continue;
                }
            }
        }
        int contains = this.trustedCerts.contains(x509Certificate);
        if (contains == 0) {
            Set<String> set = x509Certificate.getCriticalExtensionOIDs();
            if (set == null) {
                set = Collections.emptySet();
            }
            Iterator<PKIXCertPathChecker> iterator2 = forwardState.forwardCheckers.iterator();
            while (iterator2.hasNext()) {
                ((PKIXCertPathChecker)iterator2.next()).check(x509Certificate, set);
            }
            for (PKIXCertPathChecker pkixCertPathChecker : this.buildParams.certPathCheckers()) {
                if (!pkixCertPathChecker.isForwardCheckingSupported()) {
                    Set<String> supportedExtensions = pkixCertPathChecker.getSupportedExtensions();
                    if (supportedExtensions != null) {
                        set.removeAll(supportedExtensions);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            if (!set.isEmpty()) {
                set.remove(PKIXExtensions.BasicConstraints_Id.toString());
                set.remove(PKIXExtensions.NameConstraints_Id.toString());
                set.remove(PKIXExtensions.CertificatePolicies_Id.toString());
                set.remove(PKIXExtensions.PolicyMappings_Id.toString());
                set.remove(PKIXExtensions.PolicyConstraints_Id.toString());
                set.remove(PKIXExtensions.InhibitAnyPolicy_Id.toString());
                set.remove(PKIXExtensions.SubjectAlternativeName_Id.toString());
                set.remove(PKIXExtensions.KeyUsage_Id.toString());
                set.remove(PKIXExtensions.ExtendedKeyUsage_Id.toString());
                if (!set.isEmpty()) {
                    throw new CertPathValidatorException("Unrecognized critical extension(s)", null, null, -1, PKIXReason.UNRECOGNIZED_CRIT_EXT);
                }
            }
        }
        if (forwardState.isInitial()) {
            return;
        }
        if (contains == 0) {
            if (x509Certificate.getBasicConstraints() == -1) {
                throw new CertificateException("cert is NOT a CA cert");
            }
            KeyChecker.verifyCAKeyUsage(x509Certificate);
        }
        if (!forwardState.keyParamsNeeded()) {
            forwardState.cert.verify(x509Certificate.getPublicKey(), this.buildParams.sigProvider());
        }
        return;
    }
    
    @Override
    boolean isPathCompleted(X509Certificate x509Certificate) {
        ArrayList<TrustAnchor> list = new ArrayList();
        for (TrustAnchor trustAnchor : this.trustAnchors) {
            if (trustAnchor.getTrustedCert() != null) {
                if (!x509Certificate.equals(trustAnchor.getTrustedCert())) {
                    continue;
                }
                this.trustAnchor = trustAnchor;
                return true;
            }
            else {
                X500Principal ca = trustAnchor.getCA();
                PublicKey caPublicKey = trustAnchor.getCAPublicKey();
                if (ca != null && caPublicKey != null && ca.equals(x509Certificate.getSubjectX500Principal()) && caPublicKey.equals(x509Certificate.getPublicKey())) {
                    this.trustAnchor = trustAnchor;
                    return true;
                }
                list.add(trustAnchor);
            }
        }
        for (TrustAnchor trustAnchor2 : list) {
            X500Principal ca2 = trustAnchor2.getCA();
            PublicKey caPublicKey2 = trustAnchor2.getCAPublicKey();
            if (ca2 == null) {
                continue;
            }
            if (!ca2.equals(x509Certificate.getIssuerX500Principal())) {
                continue;
            }
            if (PKIX.isDSAPublicKeyWithoutParams(caPublicKey2)) {
                continue;
            }
            try {
                x509Certificate.verify(caPublicKey2, this.buildParams.sigProvider());
            }
            catch (InvalidKeyException ex) {
                if (ForwardBuilder.debug != null) {
                    ForwardBuilder.debug.println("ForwardBuilder.isPathCompleted() invalid DSA key found");
                }
                else {
                    continue;
                }
                continue;
            }
            catch (GeneralSecurityException ex2) {
                if (ForwardBuilder.debug != null) {
                    ForwardBuilder.debug.println("ForwardBuilder.isPathCompleted() unexpected exception");
                    ex2.printStackTrace();
                }
                else {
                    continue;
                }
                continue;
            }
            this.trustAnchor = trustAnchor2;
            return true;
        }
        return false;
    }
    
    @Override
    void addCertToPath(X509Certificate e, LinkedList<X509Certificate> list) {
        list.addFirst(e);
    }
    
    @Override
    void removeFinalCertFromPath(LinkedList<X509Certificate> list) {
        list.removeFirst();
    }
    
    static {
        debug = Debug.getInstance("certpath");
    }
    
    static class PKIXCertComparator implements Comparator<X509Certificate>
    {
        static final String METHOD_NME = "PKIXCertComparator.compare()";
        private final Set<X500Principal> trustedSubjectDNs;
        private final X509CertSelector certSkidSelector;
        
        PKIXCertComparator(Set<X500Principal> trustedSubjectDNs, X509CertImpl x509CertImpl) throws IOException {
            this.trustedSubjectDNs = trustedSubjectDNs;
            this.certSkidSelector = this.getSelector(x509CertImpl);
        }
        
        private X509CertSelector getSelector(X509CertImpl x509CertImpl) throws IOException {
            if (x509CertImpl != null) {
                AuthorityKeyIdentifierExtension authorityKeyIdentifierExtension = x509CertImpl.getAuthorityKeyIdentifierExtension();
                if (authorityKeyIdentifierExtension != null) {
                    byte[] encodedKeyIdentifier = authorityKeyIdentifierExtension.getEncodedKeyIdentifier();
                    if (encodedKeyIdentifier != null) {
                        X509CertSelector x509CertSelector = new X509CertSelector();
                        x509CertSelector.setSubjectKeyIdentifier(encodedKeyIdentifier);
                        return x509CertSelector;
                    }
                }
            }
            return null;
        }
        
        @Override
        public int compare(X509Certificate cert, X509Certificate x509Certificate) {
            if (cert.equals(x509Certificate)) {
                return 0;
            }
            if (this.certSkidSelector != null) {
                if (this.certSkidSelector.match(cert)) {
                    return -1;
                }
                if (this.certSkidSelector.match(x509Certificate)) {
                    return 1;
                }
            }
            X500Principal issuerX500Principal = cert.getIssuerX500Principal();
            X500Principal issuerX500Principal2 = x509Certificate.getIssuerX500Principal();
            X500Name x500Name = X500Name.asX500Name(issuerX500Principal);
            X500Name x500Name2 = X500Name.asX500Name(issuerX500Principal2);
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() o1 Issuer:  " + issuerX500Principal);
                ForwardBuilder.debug.println("PKIXCertComparator.compare() o2 Issuer:  " + issuerX500Principal2);
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() MATCH TRUSTED SUBJECT TEST...");
            }
            int contains = this.trustedSubjectDNs.contains(issuerX500Principal);
            int contains2 = this.trustedSubjectDNs.contains(issuerX500Principal2);
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() m1: " + (boolean)contains);
                ForwardBuilder.debug.println("PKIXCertComparator.compare() m2: " + (boolean)contains2);
            }
            if (contains && contains2) {
                return -1;
            }
            if (!(contains == 0)) {
                return -1;
            }
            if (!(contains2 == 0)) {
                return 1;
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() NAMING DESCENDANT TEST...");
            }
            Iterator<X500Principal> iterator = this.trustedSubjectDNs.iterator();
            while (iterator.hasNext()) {
                X500Name x500Name3 = X500Name.asX500Name((X500Principal)iterator.next());
                int distance = Builder.distance(x500Name3, x500Name, -1);
                int distance2 = Builder.distance(x500Name3, x500Name2, -1);
                if (ForwardBuilder.debug != null) {
                    ForwardBuilder.debug.println("PKIXCertComparator.compare() distanceTto1: " + distance);
                    ForwardBuilder.debug.println("PKIXCertComparator.compare() distanceTto2: " + distance2);
                }
                if (distance > 0 || distance2 > 0) {
                    if (distance == distance2) {
                        return -1;
                    }
                    if (distance > 0 && distance2 <= 0) {
                        return -1;
                    }
                    if (distance <= 0 && distance2 > 0) {
                        return 1;
                    }
                    if (distance < distance2) {
                        return -1;
                    }
                    return 1;
                }
                else {
                    continue;
                }
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() NAMING ANCESTOR TEST...");
            }
            Iterator<X500Principal> iterator2 = this.trustedSubjectDNs.iterator();
            while (iterator2.hasNext()) {
                X500Name x500Name4 = X500Name.asX500Name((X500Principal)iterator2.next());
                int distance3 = Builder.distance(x500Name4, x500Name, Integer.MAX_VALUE);
                int distance4 = Builder.distance(x500Name4, x500Name2, Integer.MAX_VALUE);
                if (ForwardBuilder.debug != null) {
                    ForwardBuilder.debug.println("PKIXCertComparator.compare() distanceTto1: " + distance3);
                    ForwardBuilder.debug.println("PKIXCertComparator.compare() distanceTto2: " + distance4);
                }
                if (distance3 < 0 || distance4 < 0) {
                    if (distance3 == distance4) {
                        return -1;
                    }
                    if (distance3 < 0 && distance4 >= 0) {
                        return -1;
                    }
                    if (distance3 >= 0 && distance4 < 0) {
                        return 1;
                    }
                    if (distance3 > distance4) {
                        return -1;
                    }
                    return 1;
                }
                else {
                    continue;
                }
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() SAME NAMESPACE AS TRUSTED TEST...");
            }
            Iterator<X500Principal> iterator3 = this.trustedSubjectDNs.iterator();
            while (iterator3.hasNext()) {
                X500Name x500Name5 = X500Name.asX500Name((X500Principal)iterator3.next());
                X500Name commonAncestor = x500Name5.commonAncestor(x500Name);
                X500Name commonAncestor2 = x500Name5.commonAncestor(x500Name2);
                if (ForwardBuilder.debug != null) {
                    ForwardBuilder.debug.println("PKIXCertComparator.compare() tAo1: " + String.valueOf(commonAncestor));
                    ForwardBuilder.debug.println("PKIXCertComparator.compare() tAo2: " + String.valueOf(commonAncestor2));
                }
                if (commonAncestor != null || commonAncestor2 != null) {
                    if (commonAncestor != null && commonAncestor2 != null) {
                        int hops = Builder.hops(x500Name5, x500Name, Integer.MAX_VALUE);
                        int hops2 = Builder.hops(x500Name5, x500Name2, Integer.MAX_VALUE);
                        if (ForwardBuilder.debug != null) {
                            ForwardBuilder.debug.println("PKIXCertComparator.compare() hopsTto1: " + hops);
                            ForwardBuilder.debug.println("PKIXCertComparator.compare() hopsTto2: " + hops2);
                        }
                        if (hops == hops2) {
                            continue;
                        }
                        if (hops > hops2) {
                            return 1;
                        }
                        return -1;
                    }
                    else {
                        if (commonAncestor == null) {
                            return 1;
                        }
                        return -1;
                    }
                }
                else {
                    continue;
                }
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() CERT ISSUER/SUBJECT COMPARISON TEST...");
            }
            X500Principal subjectX500Principal = cert.getSubjectX500Principal();
            X500Principal subjectX500Principal2 = x509Certificate.getSubjectX500Principal();
            X500Name x500Name6 = X500Name.asX500Name(subjectX500Principal);
            X500Name x500Name7 = X500Name.asX500Name(subjectX500Principal2);
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() o1 Subject: " + subjectX500Principal);
                ForwardBuilder.debug.println("PKIXCertComparator.compare() o2 Subject: " + subjectX500Principal2);
            }
            int distance5 = Builder.distance(x500Name6, x500Name, Integer.MAX_VALUE);
            int distance6 = Builder.distance(x500Name7, x500Name2, Integer.MAX_VALUE);
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() distanceStoI1: " + distance5);
                ForwardBuilder.debug.println("PKIXCertComparator.compare() distanceStoI2: " + distance6);
            }
            if (distance6 > distance5) {
                return -1;
            }
            if (distance6 < distance5) {
                return 1;
            }
            if (ForwardBuilder.debug != null) {
                ForwardBuilder.debug.println("PKIXCertComparator.compare() no tests matched; RETURN 0");
            }
            return -1;
        }
    }
}
